package fr.free.ithel.dispatcher;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.SwingUtilities;

/**
 * La classe Dispatcher est un singleton permettant de centraliser la
 * communication entre différents objets à l'aide de signaux (identifiés par des
 * String). Le fonctionnement du Dispatcher reprend le concept d'écouteurs
 * d'évènements. Ses methodes sont synchronisées, ce qui rend possible son
 * utilisation simultannée par différents threads.
 * 
 * @author ithel
 * 
 */
public class Dispatcher {

	private static Dispatcher thisDispatcher = null;

	/**
	 * Retourne une instance unique du diffuseur d'actions.
	 * 
	 * @return le diffuseur d'actions
	 */
	public static synchronized Dispatcher getInstance() {
		if (thisDispatcher == null)
			thisDispatcher = new Dispatcher();

		return thisDispatcher;
	}

	private Map<String, ArrayList<SignalListener>> map;

	private Dispatcher() {
		map = new HashMap<String, ArrayList<SignalListener>>();
	}

	/**
	 * Ajoute un ecouteur au signal.
	 * 
	 * @param signal
	 *            le signal a ecouter
	 * @param listener
	 *            l'ecouteur
	 */
	public synchronized void addSignalListener(String signal, SignalListener listener) {
		ArrayList<SignalListener> ecouteurs = map.get(signal);
		if (ecouteurs == null) {
			ecouteurs = new ArrayList<SignalListener>();
			map.put(signal, ecouteurs);
		}

		ecouteurs.add(listener);
	}

	/**
	 * Diffuse un signal a tous les SignalListener en passant par l'EDT
	 * 
	 * @param source
	 *            l'objet a l'origine de la diffusion
	 * @param signal
	 *            le signal a diffuser
	 * @param data
	 *            un <i>Object</i> a transmettre aux ecouteurs
	 */
	public synchronized void dispatchSignal(Object source, String signal, Object data) {
		final SignalEvent event = new SignalEvent(source, signal, data);
		final ArrayList<SignalListener> references = map.get(signal);
		if (references == null)
			return;

		if(!SwingUtilities.isEventDispatchThread()) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					dispatchSignal(event, references);
				}
			});
		} else
			dispatchSignal(event, references);
	}

	private void dispatchSignal(SignalEvent event, ArrayList<SignalListener> references) {
		Iterator<SignalListener> iterateur = references.iterator();
		while (iterateur.hasNext())
			iterateur.next().signalReceived(event);
	}

	/**
	 * Retourne tous les SignalListener enregistres pour ecouter un signal
	 * donne.
	 * 
	 * @param source
	 *            l'objet a l'origine de la diffusion
	 * @param signal
	 *            le signal a diffuser
	 * @param data
	 *            un Objet a transmettre aux ecouteurs
	 */
	public synchronized SignalListener[] getSignalListeners(String signal) {
		ArrayList<SignalListener> references = map.get(signal);
		SignalListener listeners[] = new SignalListener[0];
		return references == null ? listeners : references.toArray(listeners);
	}

	/**
	 * Supprime le SignalListener de la liste des ecouteurs du signal.
	 * 
	 * @param signal
	 * @param listener
	 */
	public synchronized void removeSignalListener(String signal, SignalListener listener) {
		ArrayList<SignalListener> references = map.get(signal);
		if (references == null)
			return;

		Iterator<SignalListener> iterateur = references.iterator();
		while (iterateur.hasNext()) {
			if (iterateur.next() == listener) {
				iterateur.remove();
				return;
			}
		}
	}
}
